/*---------------------------------------------------------------------------*
 |          Copyright (c) 2002-2016 PDFlib GmbH. All rights reserved.        |
 +---------------------------------------------------------------------------+
 |    This software may not be copied or distributed except as expressly     |
 |    authorized by PDFlib GmbH's general license agreement or a custom      |
 |    license agreement signed by PDFlib GmbH.                               |
 |    For more information about licensing please refer to www.pdflib.com.   |
 *---------------------------------------------------------------------------*/

/* $Id: ploplib.h,v 1.135 2016/05/15 16:43:36 rjs Exp $
 *
 * PLOP public function declarations
 *
 */

/*
 * ----------------------------------------------------------------------
 * Setup, mostly Windows calling conventions and DLL stuff
 * ----------------------------------------------------------------------
 */

#include <stdio.h>

#ifndef PLOPLIB_H_INCLUDED
#define PLOPLIB_H_INCLUDED

#if defined(WIN32) && !defined(PDFLIB_CALL)
#define PDFLIB_CALL     __cdecl
#endif

#undef PDFLIB_API
#if defined(WIN32)

    #ifdef PDFLIB_EXPORTS
    #define PDFLIB_API __declspec(dllexport) /* prepare a DLL (internal use) */

    #elif defined(PDFLIB_DLL)

    #define PDFLIB_API __declspec(dllimport) /* PDFlib clients: import  DLL */
    #endif	/* PDFLIB_DLL */

#else
    #if __GNUC__ >= 4
	#define PDFLIB_API __attribute__ ((visibility("default")))
    #elif (defined(__SUNPRO_CC) && __SUNPRO_CC >= 0x550) \
	   || (defined(__SUNPRO_C) && __SUNPRO_C >= 0x550)
	#define PDFLIB_API __global
    #endif
#endif /* WIN32 */

#ifndef PDFLIB_CALL
    #define PDFLIB_CALL	/* */	/* default: no special calling conventions */
#endif

#ifndef PDFLIB_API
    #define PDFLIB_API /* */	 /* default: generate or use static library */
#endif

/* Make our declarations C++ compatible */
#ifdef __cplusplus
extern "C" {
#endif

#define PLOP_PRODUCTNAME        "PLOP"
#define PLOP_DS_PRODUCTNAME     "PLOP DS"
#define PLOP_PRODUCTDESCR       "PDF Linearization, Optimization, Protection " \
                                "and Digital Signature"
#define PLOP_COPYRIGHT \
        "(c) 2002-2015 PDFlib GmbH  www.pdflib.com  sales@pdflib.com\n"


#define PLOP_MAJORVERSION		5
#define PLOP_MINORVERSION		1
#define PLOP_REVISION			1
#define PLOP_SHORT_VERSIONSTRING	"5"
#define PLOP_LONG_VERSIONSTRING		"5.1p1"

/* Opaque data type for the PLOP context.
*/
#if	!defined(PLOP_CONTEXT_DEFINED)
#define PLOP_CONTEXT_DEFINED
#if !defined(PLOP) || defined(ACTIVEX)
typedef struct PLOP_s PLOP;
#endif
#endif

/* The API structure with function pointers. */
typedef struct PLOP_api_s PLOP_api;

/*
 * ----------------------------------------------------------------------
 * pCOS-specific enums and defines
 * ----------------------------------------------------------------------
 */

/*
 * Guard against multiple definition of pcos_mode and pcos_object_type for the
 * case that multiple PDFlib products are used in the same program.
 */
#ifndef PDF_PCOS_ENUMS

/* document access levels.
*/
typedef enum
{
    pcos_mode_minimum	 = 0, /* encrypted doc (opened w/o password)	      */
    pcos_mode_restricted = 1, /* encrypted doc (opened w/ user password)      */
    pcos_mode_full	 = 2  /* unencrypted doc or opened w/ master password */
} pcos_mode;


/* object types.
*/
typedef enum
{
    pcos_ot_null	= 0,
    pcos_ot_boolean	= 1,
    pcos_ot_number	= 2,
    pcos_ot_name	= 3,
    pcos_ot_string	= 4,
    pcos_ot_array	= 5,
    pcos_ot_dict	= 6,
    pcos_ot_stream	= 7,
    pcos_ot_fstream	= 8
} pcos_object_type;

#define PDF_PCOS_ENUMS

#endif /* PDF_PCOS_ENUMS */

/* enums for the return values of unsupported function PLOP_get_unicode_format()
*/
typedef enum
{
    plop_uni_none     = 0,   /* no Unicode format */
    plop_uni_utf8     = 5,   /* UTF-8 */
    plop_uni_utf16    = 7,   /* UTF-16 */
    plop_uni_utf16be  = 8,   /* UTF-16 BE */
    plop_uni_utf16le  = 9,   /* UTF-16 LE */
    plop_uni_utf32    = 10   /* UTF-32 */
}
plop_unicode_format;

/*
 * ----------------------------------------------------------------------
 * General functions
 * ----------------------------------------------------------------------
 */

/* Delete a PLOP context and release all its internal resources. */
PDFLIB_API void PDFLIB_CALL
PLOP_delete(
    PLOP *plop);

/*
   Retrieve a structure with PLOP API function pointers (mainly for DLLs).
   Although this function is published here, it is not supposed to be used
   directly by clients. Use PLOP_new_dl() in ploplibdl.c instead.
 */
PDFLIB_API const PLOP_api * PDFLIB_CALL
PLOP_get_api(void);

/* Create a new PLOP context. */
PDFLIB_API PLOP * PDFLIB_CALL
PLOP_new(void);

/* Create a new PLOP context with a user-supplied error handler and
   opaque pointer. Unsupported.
 */

typedef void  (*plop_error_fp)(PLOP *plop, int type, const char *msg);

PDFLIB_API PLOP * PDFLIB_CALL
PLOP_new2(
    plop_error_fp errorhandler,
    void *opaque);

/* Fetch the opaque client pointer associated with a PLOP object. Unsupported.*/
PDFLIB_API void * PDFLIB_CALL
PLOP_get_opaque(
    PLOP *plop);

/*
 * ----------------------------------------------------------------------
 * Document input and output functions
 * ----------------------------------------------------------------------
 */

/* Add recipient for encryption with certificate security.
   Returns: -1 (in PHP: 0) on error, and 1 otherwise.
 */
PDFLIB_API int PDFLIB_CALL
PLOP_add_recipient(
    PLOP *plop,
    const char *optlist);

/* Prepare signature options.
   Returns: -1 (in PHP: 0) on error, and 1 otherwise.
 */
PDFLIB_API int PDFLIB_CALL
PLOP_prepare_signature(
    PLOP *plop,
    const char *optlist);

/* Open a PDF document (which may be protected) for processing.
   Returns: -1 (in PHP: 0) on error, and a document handle otherwise.
 */
PDFLIB_API int PDFLIB_CALL
PLOP_open_document(
    PLOP *plop,
    const char *filename,
    int len,
    const char *optlist);

/* Open a PDF document (which may be protected) via a user-supplied function.
   Returns: -1 (in PHP: 0) on error, and a document handle otherwise.
 */
PDFLIB_API int PDFLIB_CALL
PLOP_open_document_callback(
    PLOP *plop,
    void *opaque,
    size_t filesize,
    size_t (*readproc)(void *opaque, void *buffer, size_t size),
    int (*seekproc)(void *opaque, long offset),
    const char *optlist);

/* Create a PDF output document (which may be protected) in memory or on
   disk file.
   Returns: -1 (in PHP: 0) on error, and a document handle otherwise.
 */
PDFLIB_API int PDFLIB_CALL
PLOP_create_document(
    PLOP *plop,
    const char * volatile filename,
    int volatile len,
    const char *optlist);

/* Fetch full or partial buffer contents of the output document from memory.
   Returns: A buffer containing output data.
 */
PDFLIB_API const char * PDFLIB_CALL
PLOP_get_buffer(
    PLOP *plop,
    long *size);

/* Close the input and output documents. */
PDFLIB_API void PDFLIB_CALL
PLOP_close_document(
    PLOP * volatile plop,
    volatile int doc,
    const char *optlist);


/*
 * ----------------------------------------------------------------------
 * Virtual file handling
 * ----------------------------------------------------------------------
 */

/* Create a named virtual read-only file from data provided in memory. */
PDFLIB_API void PDFLIB_CALL
PLOP_create_pvf(
    PLOP *plop,
    const char *filename,
    int len,
    const void *data,
    size_t size,
    const char *optlist);

/* Delete a named virtual file and free its data structures (but not the
   contents).
   Returns: -1 if the virtual file exists but is locked, and 1 otherwise.
 */
PDFLIB_API int PDFLIB_CALL
PLOP_delete_pvf(
    PLOP *plop,
    const char *filename,
    int len);

/* Query properties of a virtual file or the PDFlib Virtual File system (PVF).
   Returns: The value of some file parameter as requested by keyword.
 */
PDFLIB_API double PDFLIB_CALL
PLOP_info_pvf(
    PLOP *plop,
    const char *filename,
    int len,
    const char *keyword);


/*
 * ----------------------------------------------------------------------
 * Option handling
 * ----------------------------------------------------------------------
 */

/* Set one or more global options for PLOP. */
PDFLIB_API void PDFLIB_CALL
PLOP_set_option(
    PLOP *plop,
    const char *optlist);


/*
 * ----------------------------------------------------------------------
 * pCOS functions
 * ----------------------------------------------------------------------
 */

/* Get the value of a pCOS path with type number or boolean.
   Returns: The numerical value of the object identified by the pCOS path.
 */
PDFLIB_API double PDFLIB_CALL
PLOP_pcos_get_number(
    PLOP *plop,
    int doc,
    const char *path, ...);

/* Get the value of a pCOS path with type name, number, string, or boolean.
   Returns: A string with the value of the object identified by the pCOS path.
 */
PDFLIB_API const char * PDFLIB_CALL
PLOP_pcos_get_string(
    PLOP *plop,
    int doc,
    const char *path, ...);

/* Get the contents of a pCOS path with type stream, fstream, or string.
   Returns: The unencrypted data contained in the stream or string.
 */
PDFLIB_API const unsigned char * PDFLIB_CALL
PLOP_pcos_get_stream(
    PLOP *plop,
    int doc,
    int *length,
    const char *optlist,
    const char *path, ...);


/*
 * ----------------------------------------------------------------------
 * Conversion function
 * ----------------------------------------------------------------------
 */

/* Convert a string in an arbitrary encoding to a Unicode string in various
 * formats.
   Returns: The converted Unicode string.
*/
PDFLIB_API const char * PDFLIB_CALL
PLOP_convert_to_unicode(
    PLOP *plop,
    const char *inputformat,
    const char *inputstring,
    int inputlen,
    int *outputlen,
    const char *optlist);

/*
 * ----------------------------------------------------------------------
 * Exception handling
 * ----------------------------------------------------------------------
 */

/* Get the number of the last thrown exception,
 * or the reason of a failed function call.
   Returns: The exceptions error number.
 */
PDFLIB_API int PDFLIB_CALL
PLOP_get_errnum(
    PLOP *plop);

/* Get the descriptive text of the last thrown exception,
   or the reason of a failed function call.
   Returns: A string describing the error, or an empty string if the last
   API call didnt cause any error.
 */
PDFLIB_API const char * PDFLIB_CALL
PLOP_get_errmsg(
    PLOP *plop);

/* Get the name of the API function which threw the last exception or failed.
   Returns: The name of a PLOP API function.
 */
PDFLIB_API const char * PDFLIB_CALL
PLOP_get_apiname(
    PLOP *plop);


/*
 * ----------------------------------------------------------------------
 * Exception handling with try/catch implementation
 * ----------------------------------------------------------------------
 */

/* Set up an exception handling frame; must always be paired with PLOP_CATCH().
*/
#define PLOP_TRY(plop)		if (setjmp(plop_jbuf(plop)->jbuf) == 0)

/* Inform the exception machinery that a PLOP_TRY() will be left without
   entering the corresponding PLOP_CATCH() clause. */

#define PLOP_EXIT_TRY(plop)	plop_exit_try(plop)

/* Catch an exception; must always be paired with PLOP_TRY(). */

#define PLOP_CATCH(plop)	if (plop_catch(plop))

/* Re-throw an exception to another handler. */

#define PLOP_RETHROW(plop)	plop_rethrow(plop)


/*
 * ----------------------------------------------------------------------
 * Deprecated functions
 * ----------------------------------------------------------------------
 */

/* Deprecated, use PLOP_convert_to_unicode() */
PDFLIB_API const char * PDFLIB_CALL
PLOP_utf8_to_utf16(
    PLOP *p,
    const char *utf8string,
    const char *ordering,
    int *size);

/* Deprecated, use PLOP_convert_to_unicode() */
PDFLIB_API const char * PDFLIB_CALL
PLOP_utf16_to_utf8(
    PLOP *p,
    const char *utf16string,
    int len,
    int *size);

/* Deprecated, use PLOP_convert_to_unicode() */
PDFLIB_API const char * PDFLIB_CALL
PLOP_utf32_to_utf8(
    PLOP *plop,
    const char *utf32string,
    int len,
    int *size);

/* Deprecated, use PLOP_convert_to_unicode() */
PDFLIB_API const char * PDFLIB_CALL
PLOP_utf32_to_utf16(
    PLOP*plop,
    const char *utf32string,
    int len,
    const char *ordering,
    int *size);

/* Deflate a Unicode string (UTF-16 or UTF-32) to a byte string (unsupported) */
PDFLIB_API const char * PDFLIB_CALL
PLOP_deflate_unicode(
    PLOP *plop,
    const char *utfstring,
    int len,
    int charsize,
    int *highchar);

/* Get the type of the Unicode format which belongs to the Unicode string
   returned from the last API function (unsupported)
*/
PDFLIB_API plop_unicode_format PDFLIB_CALL
PLOP_get_unicode_format(
    PLOP *plop);

/*
 * ----------------------------------------------------------------------
 * Private stuff, do not use explicitly but only via the above macros!
 * ----------------------------------------------------------------------
 */

#include <setjmp.h>

typedef struct
{
    jmp_buf	jbuf;
} plop_jmpbuf;

PDFLIB_API plop_jmpbuf * PDFLIB_CALL
plop_jbuf(
    PLOP *plop);

PDFLIB_API void PDFLIB_CALL
plop_exit_try(
    PLOP *plop);

PDFLIB_API int PDFLIB_CALL
plop_catch(
    PLOP *plop);

PDFLIB_API void PDFLIB_CALL
plop_rethrow(
    PLOP *plop);

PDFLIB_API void PDFLIB_CALL
plop_throw(
    PLOP *plop,
    const char *parm1,
    const char *parm2,
    const char *parm3);

/*
 * ----------------------------------------------------------------------
 * PLOP API structure with function pointers to all API functions
 * ----------------------------------------------------------------------
 */
struct PLOP_api_s
{
    /* version numbers for checking the DLL against client code */
    size_t sizeof_PLOP_api; /* size of this structure */

    int major; /* PLOP major version number */
    int minor; /* PLOP minor version number */
    int revision; /* PLOP revision number */

    int reserved; /* reserved */

    int (PDFLIB_CALL * PLOP_add_recipient)(PLOP *plop, const char *optlist);
    void (PDFLIB_CALL * PLOP_close_document)(PLOP * volatile plop,
            volatile int doc, const char *optlist);
    const char * (PDFLIB_CALL * PLOP_convert_to_unicode)(PLOP *plop,
            const char *inputformat, const char *inputstring, int inputlen,
            int *outputlen, const char *optlist);
    int (PDFLIB_CALL * PLOP_create_document)(PLOP *plop,
            const char * volatile filename,
            volatile int len, const char *optlist);
    void (PDFLIB_CALL * PLOP_create_pvf)(PLOP *plop, const char *filename,
            int len, const void *data, size_t size, const char *optlist);
    const char * (PDFLIB_CALL * PLOP_deflate_unicode)(PLOP *plop,
            const char *utfstring, int len, int charsize, int *highchar);
    void (PDFLIB_CALL * PLOP_delete)(PLOP *plop);
    int (PDFLIB_CALL * PLOP_delete_pvf)(PLOP *plop, const char *filename,
            int len);
    const PLOP_api * (PDFLIB_CALL * PLOP_get_api)(void);
    const char * (PDFLIB_CALL * PLOP_get_apiname)(PLOP *plop);
    const char * (PDFLIB_CALL * PLOP_get_buffer)(PLOP *plop, long *size);
    const char * (PDFLIB_CALL * PLOP_get_errmsg)(PLOP *plop);
    int (PDFLIB_CALL * PLOP_get_errnum)(PLOP *plop);
    void * (PDFLIB_CALL * PLOP_get_opaque)(PLOP *plop);
    plop_unicode_format (PDFLIB_CALL * PLOP_get_unicode_format)(PLOP *plop);
    double (PDFLIB_CALL * PLOP_info_pvf)(PLOP *plop, const char *filename,
            int len, const char *keyword);
    PLOP * (PDFLIB_CALL * PLOP_new)(void);
    PLOP * (PDFLIB_CALL * PLOP_new2)(plop_error_fp errorhandler, void *opaque);
    int (PDFLIB_CALL * PLOP_open_document)(PLOP *plop, const char *filename,
            int len, const char *optlist);
    int (PDFLIB_CALL * PLOP_open_document_callback)(PLOP *plop, void *opaque,
            size_t filesize, size_t(*readproc)(void *opaque, void *buffer,
                    size_t size), int(*seekproc)(void *opaque, long offset),
            const char *optlist);
    double (PDFLIB_CALL * PLOP_pcos_get_number)(PLOP *plop, int doc,
            const char *path, ...);
    const unsigned char * (PDFLIB_CALL * PLOP_pcos_get_stream)(PLOP *plop,
            int doc, int *length, const char *optlist, const char *path, ...);
    const char * (PDFLIB_CALL * PLOP_pcos_get_string)(PLOP *plop, int doc,
            const char *path, ...);
    int (PDFLIB_CALL * PLOP_prepare_signature)(PLOP *plop, const char *optlist);
    void (PDFLIB_CALL * PLOP_set_option)(PLOP *plop, const char *optlist);
    const char * (PDFLIB_CALL * PLOP_utf8_to_utf16)(PLOP *plop,
            const char *utf8string, const char *ordering, int *size);
    const char * (PDFLIB_CALL * PLOP_utf16_to_utf8)(PLOP *plop,
            const char *utf16string, int len, int *size);
    const char * (PDFLIB_CALL * PLOP_utf32_to_utf16)(PLOP *plop,
            const char *utf32string, int len, const char *ordering, int *size);
    const char * (PDFLIB_CALL * PLOP_utf32_to_utf8)(PLOP *plop,
            const char *utf32string, int len, int *size);

    plop_jmpbuf * (PDFLIB_CALL * plop_jbuf)(PLOP *plop);
    void (PDFLIB_CALL * plop_exit_try)(PLOP *plop);
    int (PDFLIB_CALL * plop_catch)(PLOP *plop);
    void (PDFLIB_CALL * plop_rethrow)(PLOP *plop);
    void (PDFLIB_CALL * plop_throw)(PLOP *plop, const char *parm1,
            const char *parm2, const char *parm3);
};

#if _MSC_VER >= 1310    /* VS .NET 2003 and later */
#pragma deprecated(PLOP_utf8_to_utf16)
#pragma deprecated(PLOP_utf16_to_utf8)
#pragma deprecated(PLOP_utf32_to_utf8)
#pragma deprecated(PLOP_utf32_to_utf16)
#pragma deprecated(PLOP_deflate_unicode)

#endif


#ifdef __cplusplus
}	/* extern "C" */
#endif

#endif	/* PLOPLIB_H_INCLUDED */
